
// LimeEdit v2.2 By CrownSoft (c) 2013
// http://www.crownsoft.net


#include"rfc_amalgamated.h"
#include"lice/lice/lice.h"
#include "ucdl.h"

#include<math.h>

#include"resource.h"

COLORREF colCitrus=RGB(230,119,11);

LICE_IBitmap *currentImage;
LICE_SysBitmap* undoImage;

KString currentDirectory;


class LimeDebugViewer : public KToolWindow, public KButtonListener, public KMenuItemListener
{
	KOverlappedWindow* parent;
	KListBox listBox;
	KMenuBar mBar;
	KMenu mFile;
	KMenuItem mClearLogs;
	KFont* listBoxFont;

	HBRUSH hbrListBox;

	static const int MENU_IMAGE_SIZE=20;
	static const int MENU_HEIGHT=22;

public:
	LimeDebugViewer(KOverlappedWindow* parent)
	{
		this->parent=parent;
		SetText(T("Debug Logs Viewer"));
		SetSize(400,350);
		SetStyle(WS_SIZEBOX|WS_OVERLAPPED|WS_SYSMENU);
		CenterScreen();

		hbrListBox=CreateSolidBrush(colCitrus);
	}

	~LimeDebugViewer()
	{
		DeleteObject(hbrListBox);
		delete listBoxFont;
	}

	void CreateSubComponents()
	{
		listBox.SetSize(392,340);
		listBox.SetPosition(0,0);
		listBox.SetStyle(listBox.GetStyle()|LBS_NOSEL);
		AddComponent(&listBox);

		mClearLogs.SetText(T("Clear All"));
		mClearLogs.SetListener(this);

		mFile.AddMenuItem(&mClearLogs);

		mBar.AddMenu(T("File"),&mFile);
		mBar.AddToWindow(this);

		ModifyMenuW(mFile.GetMenuHandle(),mClearLogs.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mClearLogs.GetItemID(),(LPCWSTR)mClearLogs.GetItemID());

		listBoxFont=new KFont(T("Lucida Console"),13,true);
		listBox.SetFont(listBoxFont);
	}

	BEGIN_KMSG_HANDLER
		ON_KMSG(WM_CTLCOLORLISTBOX,OnListBoxColor)
		ON_KMSG(WM_SIZE,OnResized) 
		ON_KMSG(WM_SIZING,OnResizing)
		ON_KMSG(WM_MEASUREITEM,OnWMMeasureItem)
		ON_KMSG(WM_DRAWITEM,OnDrawItem)
	END_KMSG_HANDLER(KToolWindow)

	LRESULT OnResizing(WPARAM wParam,LPARAM lParam)
	{
		const int MIN_WIDTH=400;
		const int MIN_HEIGHT=350;

		LPRECT lprc;
		lprc = (LPRECT) lParam;

		if((lprc->right-lprc->left)<MIN_WIDTH){
			lprc->right=lprc->left+MIN_WIDTH;
		}
		if((lprc->bottom-lprc->top)<MIN_HEIGHT){
			lprc->bottom=lprc->top+MIN_HEIGHT;
		}

		return 1;
	}

	LRESULT OnListBoxColor(WPARAM wParam,LPARAM lParam)
	{
		HDC hdcList = (HDC)wParam;

		SetTextColor(hdcList, RGB(255,255,255));
		SetBkColor(hdcList, colCitrus);

		return (LRESULT)hbrListBox;
	}

	LRESULT OnResized(WPARAM wParam,LPARAM lParam)
	{
		int nWidth = LOWORD(lParam);
		int nHeight = HIWORD(lParam);

		listBox.SetSize(nWidth-2,nHeight);

		return 0;
	}

	LRESULT OnDrawItem(WPARAM wParam,LPARAM lParam)
	{
		if(wParam==0) // it's a menu
		{
			LPDRAWITEMSTRUCT dis=(LPDRAWITEMSTRUCT)lParam;

			FillRect(dis->hDC,&dis->rcItem,(HBRUSH)GetSysColorBrush(COLOR_MENU));

			KMenuItem *item=KPlatformUtil::GetInstance()->GetMenuItemByID((UINT)dis->itemData);

			if ( (dis->itemState & ODS_SELECTED) && (dis->itemAction & (ODA_SELECT | ODA_DRAWENTIRE)) )
			{
				HBRUSH hbr=CreateSolidBrush(colCitrus);
				FillRect(dis->hDC,&dis->rcItem,hbr);
				DeleteObject(hbr);

				COLORREF oldCol=SetTextColor(dis->hDC,RGB(255,255,255));
				int oldBk=SetBkMode(dis->hDC, TRANSPARENT); 
				dis->rcItem.left+=MENU_IMAGE_SIZE;
				DrawText(dis->hDC,item->GetText(),item->GetText().GetLength(),&dis->rcItem,DT_SINGLELINE|DT_VCENTER|DT_LEFT);
				dis->rcItem.left-=MENU_IMAGE_SIZE;		
				SetTextColor(dis->hDC,oldCol);
				SetBkMode(dis->hDC, oldBk); 
			}else
			{
				dis->rcItem.left+=MENU_IMAGE_SIZE;
				DrawText(dis->hDC,item->GetText(),item->GetText().GetLength(),&dis->rcItem,DT_SINGLELINE|DT_VCENTER|DT_LEFT);
				dis->rcItem.left-=MENU_IMAGE_SIZE;
			}
			return TRUE;
		}else
		{
			return KToolWindow::WindowProc(compHWND,WM_DRAWITEM,wParam,lParam);
		}
	}

	LRESULT OnWMMeasureItem(WPARAM wParam,LPARAM lParam)
	{
		if(wParam==0) // it's a menu
		{
			LPMEASUREITEMSTRUCT mis=(LPMEASUREITEMSTRUCT)lParam;
			mis->itemHeight=MENU_HEIGHT;
			HDC hdc=GetDC(compHWND);
			HGDIOBJ hfontOld = SelectObject(hdc,(HFONT)GetStockObject(DEFAULT_GUI_FONT));
			KMenuItem *item=KPlatformUtil::GetInstance()->GetMenuItemByID((UINT)mis->itemData);
			SIZE sz={0,0};
			GetTextExtentPoint32W(hdc,item->GetText(),item->GetText().GetLength(),&sz);
			mis->itemWidth=sz.cx+MENU_IMAGE_SIZE;
			SelectObject(hdc,hfontOld);
			ReleaseDC(compHWND,hdc);

			return 1;
		}else
		{
			return KToolWindow::WindowProc(compHWND,WM_MEASUREITEM,wParam,lParam);
		}
	}


	 void OnMenuItemPress(KMenuItem *menuItem)
	 {
		 if(menuItem==&mClearLogs)
		 {
			listBox.ClearList();
		 }
	 }

	void DebugOutString(char* text)
	{
		int len=(int)strlen(text);
		if(len>0)
		{
			//remove newlines...
			for(int i=0;i<len;i++)
			{
				if(text[i]=='\n')
				{
					text[i]=' ';
				}
			}

			listBox.AddItem(KString(text));
		}
	}

	void DebugOutInt(int num)
	{
		KString txt(num,10);
		listBox.AddItem(txt);
	}

	void OnClose()
	{
		SetVisible(false);
		parent->BringToFront();
	}
};

LimeDebugViewer* limeDebugViewer=0;

//==========script apis============================

__declspec(dllexport) void LineTo(void* image,int x1, int y1, int x2, int y2, int col, float alpha)
{
	LICE_Line((LICE_SysBitmap*)image,x1,y1,x2,y2,col,alpha,LICE_BLIT_MODE_COPY,true);
}

__declspec(dllexport) void Blur(void* image,int amount,int x,int y,int w,int h)
{
	for(int i=0;i<amount;i++)
	{
		LICE_Blur((LICE_SysBitmap*)image,(LICE_SysBitmap*)image,x,y,x,y,w,h);
	}
}

__declspec(dllexport) void FillConvexPolygon(void* image,int* x, int* y, int npoints, int col, float alpha)
{
	LICE_FillConvexPolygon((LICE_SysBitmap*)image,x,y,npoints,col,alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) void FillCircle(void* image,float cx, float cy, float r, int col, float alpha)
{
	LICE_FillCircle((LICE_SysBitmap*)image,cx, cy, r, col, alpha,LICE_BLIT_MODE_COPY,true);
}

__declspec(dllexport) void DrawCircle(void* image,float cx, float cy, float r, int col, float alpha)
{
	LICE_Circle((LICE_SysBitmap*)image,cx,cy,r,col,alpha,LICE_BLIT_MODE_COPY,true);
}

__declspec(dllexport) void DrawArc(void* image,float cx,float cy,float r,float minAngle,float maxAngle,int col,float alpha)
{
	LICE_Arc((LICE_SysBitmap*)image,cx,cy,r,minAngle,maxAngle,col,alpha,LICE_BLIT_MODE_COPY,true);
}

__declspec(dllexport) void FillTriangle(void* image,int x1, int y1, int x2, int y2, int x3, int y3, int col, float alpha)
{
	LICE_FillTriangle((LICE_SysBitmap*)image,x1,y1,x2,y2,x3,y3,col,alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) void FillRectangle(void* image,int x,int y,int w,int h,int col,float alpha)
{
	LICE_FillRect((LICE_SysBitmap*)image,x,y,w,h,col,alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) void DrawRectangle(void* image,int x, int y, int w, int h, int col, float alpha)
{
	LICE_DrawRect((LICE_SysBitmap*)image, x, y, w, h, col, alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) int GetPixel(void* image,int x,int y)
{
	return LICE_GetPixel((LICE_SysBitmap*)image,x,y);
}

__declspec(dllexport) void SetPixel(void* image,int x,int y,int col,float alpha)
{
	LICE_PutPixel((LICE_SysBitmap*)image,x,y,col,alpha,LICE_BLIT_MODE_COPY);
}

__declspec(dllexport) int GetWidth(void* image)
{
	return ((LICE_SysBitmap*)image)->getWidth();
}

__declspec(dllexport) int GetHeight(void* image)
{
 return ((LICE_SysBitmap*)image)->getHeight();
}

__declspec(dllexport) void DebugOutString(char* text)
{
	limeDebugViewer->DebugOutString(text);
}

__declspec(dllexport) void DebugOutInt(int num)
{
	limeDebugViewer->DebugOutInt(num);
}

__declspec(dllexport) void* LimeAlloc(int size)
{
	return malloc(size);
}

__declspec(dllexport) void LimeFree(void* buff)
{
	free(buff);
}


//=========== v2.0 new apis ======================

__declspec(dllexport) void* CreateImage(int w,int h)
{
	return new LICE_SysBitmap(w,h);
}

__declspec(dllexport) void DeleteImage(void* image)
{
	delete (LICE_SysBitmap*)image;
}

__declspec(dllexport) void* GetCurrentImage()
{
	return (LICE_SysBitmap*)currentImage;
}

__declspec(dllexport) void SetCurrentImage(void* image)
{
	currentImage=(LICE_SysBitmap*)image;
}

__declspec(dllexport) void DrawImage(void* dest,int destX,int destY,int destW,int destH,void* src,int srcX,int srcY,int srcW,int srcH,float alpha)
{
	LICE_ScaledBlit((LICE_SysBitmap*)dest,(LICE_SysBitmap*)src,destX,destY,destW,destH,srcX,srcY,srcW,srcH,alpha,LICE_BLIT_USE_ALPHA);
}


__declspec(dllexport) void* LoadPNGImage(char* path)
{
	LICE_SysBitmap *image=new LICE_SysBitmap(5,5);
	LICE_LoadPNG(path,image);
	return (LICE_SysBitmap*)image;
}

__declspec(dllexport) void SavePNGImage(void* image,char* path)
{
	LICE_WritePNG(path,(LICE_SysBitmap*)image);
}

__declspec(dllexport) void* LoadJPGImage(char* path)
{
	LICE_SysBitmap *image=new LICE_SysBitmap(5,5);
	LICE_LoadJPG(path,image);
	return (LICE_SysBitmap*)image;
}

__declspec(dllexport) void SaveJPGImage(void* image,char* path)
{
	LICE_WriteJPG(path,(LICE_SysBitmap*)image);
}

__declspec(dllexport) void WriteToFile(void* buffer,int size,char* path)
{
	HANDLE fileHandle=CreateFileA(path,GENERIC_WRITE,FILE_SHARE_READ,0,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
	if(fileHandle!=INVALID_HANDLE_VALUE)
	{
		DWORD bWritten;
		WriteFile(fileHandle,buffer,size,&bWritten,0);
		CloseHandle(fileHandle);
	}
}

__declspec(dllexport) void RGB2HSV(int r, int g, int b, int* h, int* s, int* v)
{
	LICE_RGB2HSV(r,g,b,h,s,v);
}

__declspec(dllexport) void HSV2RGB(int h, int s, int v, int* r, int* g, int* b)
{
	LICE_HSV2RGB(h,s,v,r,g,b);
}

//=========== v2.1 new apis ======================

__declspec(dllexport) void* WriteGifBegin(char* path, int w, int h)
{
  return LICE_WriteGIFBeginNoFrame(path,w,h);
}

__declspec(dllexport) void WriteGifFrame(void* gifHandle,void* image,int delay)
{
	LICE_WriteGIFFrame(gifHandle,(LICE_SysBitmap*)image,0,0,true,delay);
}

__declspec(dllexport) void WriteGifEnd(void* gifHandle)
{
	LICE_WriteGIFEnd(gifHandle);
}

//=========== v2.2 new apis ======================

__declspec(dllexport) void* MakeFont(char* fontName,int size,bool bold,bool italic)
{
	return new KFont(KString(fontName),size,bold,italic);
}

__declspec(dllexport) void DeleteFont(void* font)
{
	delete (KFont*)font;
}


// DrawText does not use alpha channel, so we create seperate layer for text and apply alpha
__declspec(dllexport) void DrawString(void* image,void* font,char* text,int x,int y,int w,int h,int col,bool vcenter,bool hcenter)
{
	KFont *fnt=(KFont*)font;
	LICE_SysBitmap *bmp=(LICE_SysBitmap*)image;
	RECT rect={x,y,x+w,y+h};
	COLORREF color=RGB(LICE_GETR(col),LICE_GETG(col),LICE_GETB(col));

	LICE_SysBitmap *layer=new LICE_SysBitmap(bmp->getWidth(),bmp->getHeight());
	LICE_pixel layerCol=LICE_RGBA((LICE_GETR(col)+10)>255?0:(LICE_GETR(col)+10),0,0,0); // fill transparent layer with different color than font color 
	LICE_FillRect(layer,0,0,bmp->getWidth(),bmp->getHeight(),layerCol,1.0f,LICE_BLIT_MODE_COPY);

	HDC hdc=layer->getDC();

	int oldBkMode=SetBkMode(hdc,TRANSPARENT);
	HGDIOBJ hOldFont=SelectObject(hdc,fnt->GetFontHandle());
	COLORREF oldCol=SetTextColor(hdc,color);

	UINT format=0;
	if(vcenter)
		format=format|DT_VCENTER|DT_SINGLELINE|DT_NOPREFIX;
	if(hcenter)
		format=format|DT_CENTER|DT_NOPREFIX;

	if(!format)
		format=DT_EDITCONTROL|DT_WORDBREAK|DT_NOPREFIX;

	DrawTextA(hdc,text,-1,&rect,format);
	//TextOutA(hdc,x,y,text,strlen(text));

	SetBkMode(hdc,oldBkMode);
	SetTextColor(hdc,oldCol);
	SelectObject(hdc,hOldFont);

	// apply alpha to text pixels
	for(int i=x;i<layer->getWidth();i++)
	{
		for(int j=y;j<layer->getHeight();j++)
		{
			LICE_pixel pix=LICE_GetPixel(layer,i,j);
			if( pix!=layerCol)
			{
				LICE_PutPixel(layer,i,j,col,1.0f,LICE_BLIT_MODE_COPY);
			}
		}
	}

	LICE_Blit(bmp,layer,0,0,0,0,layer->getWidth(),layer->getHeight(),1.0f,LICE_BLIT_USE_ALPHA);

	delete layer;

}

//================================================


class LimeTextArea : public KTextArea
{
public:

	BEGIN_KMSG_HANDLER
		ON_KMSG(WM_SETFOCUS ,OnSetFocus)
	END_KMSG_HANDLER(KTextArea)

	LRESULT OnSetFocus(WPARAM wParam,LPARAM lParam) // caret is solid ;-)
	{
		LRESULT retVal=KTextArea::WindowProc(compHWND,WM_SETFOCUS,wParam,lParam);
		CreateCaret(compHWND,0,4,13);
		ShowCaret(compHWND);
		return retVal;
	}

};

class LimeScriptEditor : public KToolWindow, public KButtonListener
{
	KOverlappedWindow* parent;
	LimeTextArea textArea;
	KButton btnBrowse,btnSave,btnExec,btnUndo;
	KFont* scriptFont;
	HBRUSH hbrEdit;

	
public:
	LimeScriptEditor(KOverlappedWindow* parent)
	{
		this->parent=parent;
		SetText(T("Lime Script Editor"));
		SetSize(400,350);
		SetStyle(WS_SIZEBOX|WS_OVERLAPPED|WS_SYSMENU);
		CenterScreen();

		hbrEdit=CreateSolidBrush(colCitrus);
		
	}

	~LimeScriptEditor()
	{
		DeleteObject(hbrEdit);
		delete scriptFont;
	}

	BEGIN_KMSG_HANDLER
		ON_KMSG(WM_CTLCOLOREDIT,OnEditBoxColor)
		ON_KMSG(WM_SIZE,OnResized) 
		ON_KMSG(WM_SIZING,OnResizing)
	END_KMSG_HANDLER(KToolWindow)

	LRESULT OnEditBoxColor(WPARAM wParam,LPARAM lParam)
	{
		HDC hdcEdit = (HDC)wParam;

		SetTextColor(hdcEdit, RGB(255,255,255));
		SetBkColor(hdcEdit, colCitrus);

		return (LRESULT)hbrEdit;
	}

	LRESULT OnResized(WPARAM wParam,LPARAM lParam)
	{
		int nWidth = LOWORD(lParam);
		int nHeight = HIWORD(lParam);

		textArea.SetSize(nWidth-12,nHeight-60);

		btnBrowse.SetPosition(8,nHeight-36);
		btnSave.SetPosition(83,nHeight-36);
		btnExec.SetPosition(nWidth-89,nHeight-36);
		btnUndo.SetPosition(nWidth-175,nHeight-36);

		return 0;
	}

	LRESULT OnResizing(WPARAM wParam,LPARAM lParam)
	{
		const int MIN_WIDTH=400;
		const int MIN_HEIGHT=350;

		LPRECT lprc;
		lprc = (LPRECT) lParam;

		if((lprc->right-lprc->left)<MIN_WIDTH){
			lprc->right=lprc->left+MIN_WIDTH;
		}
		if((lprc->bottom-lprc->top)<MIN_HEIGHT){
			lprc->bottom=lprc->top+MIN_HEIGHT;
		}

		return 1;
	}

	void CreateSubComponents()
	{
		textArea.SetSize(380,265);
		textArea.SetPosition(8,10);
		AddComponent(&textArea);

		btnBrowse.SetText(T("Browse..."));
		btnBrowse.SetPosition(8,290);
		btnBrowse.SetSize(70,25);
		btnBrowse.SetListener(this);
		AddComponent(&btnBrowse);

		btnSave.SetText(T("Save As..."));
		btnSave.SetPosition(83,290);
		btnSave.SetSize(70,25);
		btnSave.SetListener(this);
		AddComponent(&btnSave);

		btnExec.SetText(T("Execute"));
		btnExec.SetPosition(308,290);
		btnExec.SetSize(80,25);
		btnExec.SetListener(this);
		AddComponent(&btnExec);

		btnUndo.SetText(T("Undo"));
		btnUndo.SetPosition(216,290);
		btnUndo.SetSize(80,25);
		btnUndo.SetListener(this);
		AddComponent(&btnUndo);

		scriptFont=new KFont(T("Lucida Console"),13,true);
		textArea.SetFont(scriptFont);
	}

	 void OnButtonPress(KButton *button)
	 {
		if(button==&btnExec)
		{
			if( (currentImage!=0) && (!textArea.GetText().IsEmpty()) )
			{
				LICE_Copy(undoImage,currentImage);
				KString txt=textArea.GetText();
				int result=uc_exec((char*)(const char*)txt);
				if(result==0) // error in syntax!
				{
					char buff[256];
					uc_error(buff,256);
					limeDebugViewer->DebugOutString(buff);
					Beep(256,100);
				}
				parent->Repaint();
			}
		}else if(button==&btnBrowse)
		{

			KString fileName;
			if(KCommonDialogBox::ShowOpenFileDialog(this,T("Open Script..."),L"Script Files (*.lms)\0*.lms\0",&fileName))
			{
				HANDLE hFile=CreateFile(fileName,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
				DWORD fileSize=GetFileSize(hFile,0);
				if(fileSize)
				{
					char* buffer=(char*)malloc(fileSize);
					DWORD lpNumberOfBytesRead;
					ReadFile(hFile,buffer,fileSize,&lpNumberOfBytesRead,0);
					CloseHandle(hFile);
					
					textArea.SetText(KString(buffer));
					free(buffer);

					SetCurrentDirectoryW(currentDirectory);
				}
			}
		}else if(button==&btnSave)
		{
			if(!textArea.GetText().IsEmpty())
			{
				KString fileName;
				if(KCommonDialogBox::ShowSaveFileDialog(this,T("Save Script..."),L"Script Files (*.lms)\0*.lms\0",&fileName))
				{
					HANDLE hFile=CreateFile(fileName.Append(".lms"),GENERIC_WRITE,FILE_SHARE_READ,0,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
					KString txt=textArea.GetText();
					char* buffer=(char*)(const char*)txt;
					DWORD size=textArea.GetText().GetLength()+1;
					DWORD lpNumberOfBytesWritten;
					WriteFile(hFile,buffer,size,&lpNumberOfBytesWritten,0);
					CloseHandle(hFile);

					SetCurrentDirectoryW(currentDirectory);
				}
			}
		}else if(button==&btnUndo)
		{
			LICE_Copy(currentImage,undoImage);
			parent->Repaint();
		}

	 }

	void OnClose()
	{
		SetVisible(false);
		parent->BringToFront();
	}
};


class NewImageListener
{
public:
	virtual void OnNewImageCreate(int w,int h)=0;
};

class LimeEditNewImageDialog: public KToolWindow, public KButtonListener
{
	KButton btnCreate;
	KLabel lblWidth,lblHeight;
	KNumericField txtWidth,txtHeight;
public:
	NewImageListener* listener;

	LimeEditNewImageDialog()
	{
		listener=0;
		SetSize(200,130);
		SetVisible(false);
		SetText(T("New Image..."));
	}

	void CreateNewImageDialog()
	{
		CreateComponent();
		CenterScreen();

		lblWidth.SetText(T("Width"));
		lblHeight.SetText(T("Height"));
		txtWidth.SetText(T("400"));
		txtHeight.SetText(T("200"));
		btnCreate.SetText(T("Create"));

		lblWidth.SetSize(50,25);
		lblHeight.SetSize(50,25);
		btnCreate.SetSize(50,25);

		lblWidth.SetPosition(10,13);
		lblHeight.SetPosition(10,43);
		txtWidth.SetPosition(80,10);
		txtHeight.SetPosition(80,40);
		btnCreate.SetPosition(130,70);

		btnCreate.SetListener(this);

		AddComponent(&lblWidth);
		AddComponent(&lblHeight);
		AddComponent(&txtWidth);
		AddComponent(&txtHeight);
		AddComponent(&btnCreate);
	}

	 void OnButtonPress(KButton *button)
	 {
		if(button==&btnCreate)
		{
			if(txtWidth.GetText().GetLength())
			{
				if(txtHeight.GetText().GetLength())
				{
					int w=txtWidth.GetText().GetIntValue();
					int h=txtHeight.GetText().GetIntValue();
					if( (w>0) && (h>0))
					{
						SetVisible(false);
						if(listener)
							listener->OnNewImageCreate(w,h);
					}
				}
			}
		}
	 }

	void OnClose()
	{
		SetVisible(false);
		BringWindowToTop(compParentHWND);
	}
};

class LimeEditMainWindow : public KOverlappedWindow, public KMenuItemListener,public NewImageListener
{
	KMenuBar mBar;
	KMenu mFile,mEdit,mScript,mHelp;
	KMenuItem mNew,mOpen,mSavePNG,mSaveJPG,mExit,mUndo,mScriptEdit,mDebugViewer,mManual,mAbout;
	LICE_SysBitmap *backBuffer;

	LimeEditNewImageDialog newImageDialog;

	LimeScriptEditor* scriptEditor;
	KIcon limeIcon;

	static const int MENU_IMAGE_SIZE=20;
	static const int MENU_HEIGHT=22;

public:

	LimeEditMainWindow()
	{
		newImageDialog.listener=this;

		SetText(T("LimeEdit v2.2"));
		SetSize(800,600);

		currentImage=0;
		backBuffer=new LICE_SysBitmap(compWidth ,compHeight);

		undoImage=new LICE_SysBitmap(10,10);

		limeIcon.LoadFromResource(IDI_ICON1);
	}

	~LimeEditMainWindow()
	{
		if(currentImage)
			delete currentImage;
		delete backBuffer;
		delete undoImage;
		delete scriptEditor;
		delete limeDebugViewer;
	}

	void OnNewImageCreate(int w,int h)
	{
		if(currentImage)
			delete currentImage;

		currentImage=new LICE_SysBitmap(w,h);
		LICE_FillRect(currentImage,0,0,w,h,LICE_RGBA(255,255,255,255),1.0,LICE_BLIT_MODE_COPY);
		LICE_Copy(undoImage,currentImage);

		SetCurrentDirectoryW(currentDirectory);

		this->Repaint();
	}

	void CreateElements()
	{
		CreateComponent();
		SetIcon(&limeIcon);

		mNew.SetText(T("New Image..."));
		mOpen.SetText(T("Open Image..."));
		mSavePNG.SetText(T("Save Image As (PNG)..."));
		mSaveJPG.SetText(T("Save Image As (JPG)..."));
		mExit.SetText(T("Exit"));
		mUndo.SetText(T("Undo Last Action"));
		mScriptEdit.SetText(T("Script Editor..."));
		mDebugViewer.SetText(T("View Debug Logs..."));
		mManual.SetText(T("LimeEdit Manual"));
		mAbout.SetText(T("About LimeEdit..."));

		mNew.SetListener(this);
		mOpen.SetListener(this);
		mSavePNG.SetListener(this);
		mSaveJPG.SetListener(this);
		mExit.SetListener(this);
		mUndo.SetListener(this);
		mScriptEdit.SetListener(this);
		mDebugViewer.SetListener(this);
		mManual.SetListener(this);
		mAbout.SetListener(this);

		mFile.AddMenuItem(&mNew);
		mFile.AddSeperator();
		mFile.AddMenuItem(&mOpen);
		mFile.AddMenuItem(&mSavePNG);
		mFile.AddMenuItem(&mSaveJPG);
		mFile.AddSeperator();
		mFile.AddMenuItem(&mExit);

		mEdit.AddMenuItem(&mUndo);

		mScript.AddMenuItem(&mScriptEdit);
		mScript.AddSeperator();
		mScript.AddMenuItem(&mDebugViewer);

		mHelp.AddMenuItem(&mManual);
		mHelp.AddSeperator();
		mHelp.AddMenuItem(&mAbout);

		mBar.AddMenu(T("File"),&mFile);
		mBar.AddMenu(T("Edit"),&mEdit);
		mBar.AddMenu(T("Script"),&mScript);
		mBar.AddMenu(T("Help"),&mHelp);
		mBar.AddToWindow(this);

		ModifyMenuW(mFile.GetMenuHandle(),mNew.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mNew.GetItemID(),(LPCWSTR)mNew.GetItemID());
		ModifyMenuW(mFile.GetMenuHandle(),mOpen.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mOpen.GetItemID(),(LPCWSTR)mOpen.GetItemID());
		ModifyMenuW(mFile.GetMenuHandle(),mSavePNG.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mSavePNG.GetItemID(),(LPCWSTR)mSavePNG.GetItemID());
		ModifyMenuW(mFile.GetMenuHandle(),mSaveJPG.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mSaveJPG.GetItemID(),(LPCWSTR)mSaveJPG.GetItemID());
		ModifyMenuW(mFile.GetMenuHandle(),mExit.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mExit.GetItemID(),(LPCWSTR)mExit.GetItemID());
		ModifyMenuW(mEdit.GetMenuHandle(),mUndo.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mUndo.GetItemID(),(LPCWSTR)mUndo.GetItemID());		
		ModifyMenuW(mScript.GetMenuHandle(),mScriptEdit.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mScriptEdit.GetItemID(),(LPCWSTR)mScriptEdit.GetItemID());
		ModifyMenuW(mScript.GetMenuHandle(),mDebugViewer.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mDebugViewer.GetItemID(),(LPCWSTR)mDebugViewer.GetItemID());		
		ModifyMenuW(mHelp.GetMenuHandle(),mManual.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mManual.GetItemID(),(LPCWSTR)mManual.GetItemID());
		ModifyMenuW(mHelp.GetMenuHandle(),mAbout.GetItemID(),MF_BYCOMMAND|MF_OWNERDRAW,mAbout.GetItemID(),(LPCWSTR)mAbout.GetItemID());
	
		scriptEditor=new LimeScriptEditor(this);
		scriptEditor->SetParentHWND(compHWND);
		scriptEditor->CreateComponent();
		scriptEditor->CreateSubComponents();

		limeDebugViewer=new LimeDebugViewer(this);
		limeDebugViewer->SetParentHWND(compHWND);
		limeDebugViewer->CreateComponent();
		limeDebugViewer->CreateSubComponents();

		newImageDialog.SetParentHWND(compHWND);
		newImageDialog.CreateNewImageDialog();
	}

	void OnMenuItemPress(KMenuItem *menuItem)
	{
		if(menuItem==&mNew)
		{
			newImageDialog.SetVisible(true);
		}else if(menuItem==&mOpen)
		{
			KString fileName;
			if(KCommonDialogBox::ShowOpenFileDialog(this,T("Open Image..."),L"Image Files (PNG or JPG)\0*.png;*.jpg\0",&fileName))
			{
				if(currentImage)
					delete currentImage;

				currentImage=new LICE_SysBitmap(10,10); // create sysbmp to pass loadimage, otherwise loadimage will return membitmap instead of sysbmp
				LICE_LoadImage(fileName,currentImage);//LICE_LoadPNG(fileName);
				LICE_Copy(undoImage,currentImage);

				SetCurrentDirectoryW(currentDirectory);

				this->Repaint();
			}	
		}else if(menuItem==&mSavePNG)
		{
			if(currentImage)
			{
				KString fileName;
				if(KCommonDialogBox::ShowSaveFileDialog(this,T("Save Image As..."),L"Image Files (*.png)\0*.png\0",&fileName))
				{
					LICE_WritePNG(fileName.Append(".png"),currentImage);

					SetCurrentDirectoryW(currentDirectory);
				}
			}
		}else if(menuItem==&mSaveJPG)
		{
			if(currentImage)
			{
				KString fileName;
				if(KCommonDialogBox::ShowSaveFileDialog(this,T("Save Image As..."),L"Image Files (*.jpg)\0*.jpg\0",&fileName))
				{
					LICE_WriteJPG(fileName.Append(".jpg"),currentImage);

					SetCurrentDirectoryW(currentDirectory);
				}
			}
		}else if(menuItem==&mExit)
		{
			SendMessage(compHWND,WM_CLOSE,0,0);
		}else if(menuItem==&mUndo)
		{
			if(currentImage)
			{
				LICE_Copy(currentImage,undoImage);
				Repaint();
			}
		}else if(menuItem==&mScriptEdit)
		{	
			if(currentImage)
			{
				scriptEditor->SetVisible(true);
			}
		}else if(menuItem==&mDebugViewer)
		{
			if(currentImage)
			{
				limeDebugViewer->SetVisible(true);
			}
		}else if(menuItem==&mManual)
		{
			ShellExecuteW(compHWND,L"open",L"manual.txt",0,0,SW_SHOWNORMAL);
		}
		else if(menuItem==&mAbout)
		{
			MSGBOXPARAMS msgparams;
			ZeroMemory(&msgparams,sizeof(MSGBOXPARAMS));
			msgparams.cbSize=sizeof(MSGBOXPARAMS);
			msgparams.hwndOwner=compHWND;
			msgparams.hInstance=KPlatformUtil::GetInstance()->GetAppInstance();
			msgparams.dwStyle=MB_OK|MB_USERICON;
			msgparams.lpszIcon=MAKEINTRESOURCEW(IDI_ICON1);
			msgparams.lpszText=L"LimeEdit v2.2\nhttp://www.crownsoft.net\n\nCreated with:\n\tMicrosoft C++ Compiler v8.0\n\tLibPNG version 1.2.16 \n\tJPEG Lib version 6b\n\tGifLib version r94\n\tZLib version 1.2.3\n\tCockos WDL - LICE\n\tUnderC version 1.2.9w\n\tRFC Framework v0.2.1\n\nLICE, ZLib, LibPNG, JPEG Lib, GifLib and UnderC belogs to their respective owners.\nThis program is freeware and released under the GNU General Public License.";
			msgparams.lpszCaption=L"About";
			MessageBoxIndirect(&msgparams);
		}
	}

	BEGIN_KMSG_HANDLER
		ON_KMSG(WM_PAINT,OnPaint)
		ON_KMSG(WM_ERASEBKGND,OnEraseBack)
		ON_KMSG(WM_SIZING,OnResizing)
		ON_KMSG(WM_MEASUREITEM,OnWMMeasureItem)
		ON_KMSG(WM_DRAWITEM,OnDrawItem)
	END_KMSG_HANDLER(KOverlappedWindow)

	LRESULT OnDrawItem(WPARAM wParam,LPARAM lParam)
	{
		if(wParam==0) // it's a menu
		{
			LPDRAWITEMSTRUCT dis=(LPDRAWITEMSTRUCT)lParam;

			FillRect(dis->hDC,&dis->rcItem,(HBRUSH)GetSysColorBrush(COLOR_MENU));

			KMenuItem *item=KPlatformUtil::GetInstance()->GetMenuItemByID((UINT)dis->itemData);

			if ( (dis->itemState & ODS_SELECTED) && (dis->itemAction & (ODA_SELECT | ODA_DRAWENTIRE)) )
			{
				HBRUSH hbr=CreateSolidBrush(colCitrus);
				FillRect(dis->hDC,&dis->rcItem,hbr);
				DeleteObject(hbr);

				COLORREF oldCol=SetTextColor(dis->hDC,RGB(255,255,255));
				int oldBk=SetBkMode(dis->hDC, TRANSPARENT); 
				dis->rcItem.left+=MENU_IMAGE_SIZE;
				DrawText(dis->hDC,item->GetText(),item->GetText().GetLength(),&dis->rcItem,DT_SINGLELINE|DT_VCENTER|DT_LEFT);
				dis->rcItem.left-=MENU_IMAGE_SIZE;		
				SetTextColor(dis->hDC,oldCol);
				SetBkMode(dis->hDC, oldBk); 
			}else
			{
				dis->rcItem.left+=MENU_IMAGE_SIZE;
				DrawText(dis->hDC,item->GetText(),item->GetText().GetLength(),&dis->rcItem,DT_SINGLELINE|DT_VCENTER|DT_LEFT);
				dis->rcItem.left-=MENU_IMAGE_SIZE;
			}
			return TRUE;
		}else
		{
			return KOverlappedWindow::WindowProc(compHWND,WM_DRAWITEM,wParam,lParam);
		}
	}

	LRESULT OnWMMeasureItem(WPARAM wParam,LPARAM lParam)
	{
		if(wParam==0) // it's a menu
		{
			LPMEASUREITEMSTRUCT mis=(LPMEASUREITEMSTRUCT)lParam;
			mis->itemHeight=MENU_HEIGHT;
			HDC hdc=GetDC(compHWND);
			HGDIOBJ hfontOld = SelectObject(hdc,(HFONT)GetStockObject(DEFAULT_GUI_FONT));
			KMenuItem *item=KPlatformUtil::GetInstance()->GetMenuItemByID((UINT)mis->itemData);
			SIZE sz={0,0};
			GetTextExtentPoint32W(hdc,item->GetText(),item->GetText().GetLength(),&sz);
			mis->itemWidth=sz.cx+MENU_IMAGE_SIZE;
			SelectObject(hdc,hfontOld);
			ReleaseDC(compHWND,hdc);

			return 1;
		}else
		{
			return KOverlappedWindow::WindowProc(compHWND,WM_MEASUREITEM,wParam,lParam);
		}
	}

	LRESULT OnResizing(WPARAM wParam,LPARAM lParam)
	{
		const int MIN_WIDTH=400;
		const int MIN_HEIGHT=400;

		LPRECT lprc;
		lprc = (LPRECT) lParam;

		if((lprc->right-lprc->left)<MIN_WIDTH){
			lprc->right=lprc->left+MIN_WIDTH;
		}
		if((lprc->bottom-lprc->top)<MIN_HEIGHT){
			lprc->bottom=lprc->top+MIN_HEIGHT;
		}

		return 1;
	}

	LRESULT OnEraseBack(WPARAM wParam,LPARAM lParam)
	{
		return 1;
	}


	LRESULT OnPaint(WPARAM wParam,LPARAM lParam)
	{
		HDC hdc;
		PAINTSTRUCT ps;
		RECT rect;
		hdc=BeginPaint(compHWND,&ps);
		GetClientRect(compHWND,&rect);

		int clientWidth=rect.right-rect.left;
		int clientHeight=rect.bottom-rect.top;

		backBuffer->resize(clientWidth ,clientHeight);

		LICE_FillRect(backBuffer,0,0,clientWidth,clientHeight,LICE_RGBA(128,128,128,255));

		if(currentImage) // keep image ratio with window size!
		{
			int imgWidth=currentImage->getWidth();
			int imgHeight=currentImage->getHeight();
			if( (imgWidth>clientWidth) || (imgHeight>clientHeight) )
			{
				int newImgHeight,newImgWidth;

				if(imgHeight>imgWidth)
				{
					int i=20;
					while(true)
					{
						newImgHeight=clientHeight-i;
						newImgWidth=((float)newImgHeight/(float)imgHeight)*(float)imgWidth;	
						if( newImgWidth < (clientWidth-20) )
						{
							break;
						}
						i++;
					}
				}else
				{
					int i=20;
					while(true)
					{
						newImgWidth=clientWidth-i;
						newImgHeight=((float)newImgWidth/(float)imgWidth)*(float)imgHeight;	
						if( newImgHeight < (clientHeight-20) )
						{
							break;
						}
						i++;
					}
				}

				LICE_ScaledBlit(backBuffer,currentImage,(clientWidth-newImgWidth)/2,(clientHeight-newImgHeight)/2,newImgWidth,newImgHeight,0,0,imgWidth,imgHeight,1.0,LICE_BLIT_USE_ALPHA);

			}else
			{
				LICE_Blit(backBuffer,currentImage,(clientWidth-imgWidth)/2,(clientHeight-imgHeight)/2,0,0,imgWidth,imgHeight,1.0,LICE_BLIT_USE_ALPHA);
			}
		}
		
		BitBlt(hdc,0,0,clientWidth ,clientHeight,backBuffer->getDC(),0,0,SRCCOPY);	
		
		EndPaint(compHWND,&ps);

		return 0;
	}

};

class MyApp : public KApplication
{
public:
	int Main(KString **argv,int argc)
	{
		// setup path...
		static wchar_t path[MAX_PATH];
		GetModuleFileNameW(KPlatformUtil::GetInstance()->GetAppInstance(),path,MAX_PATH);

		wchar_t *p;
		for (p = path; *p; p++) { }	// find end
		for (; p > path && *p != '\\'; p--) { } // back up to last backslash
		*p = 0;	// kill it

		currentDirectory=path;

		uc_init(0,0);
		uc_include("lime.h");

		LimeEditMainWindow wnd;

		wnd.CenterScreen();
		wnd.SetVisible(true);
		wnd.CreateElements();
		wnd.BringToFront();

		DoMessagePump(false);

		uc_finis();

		return 0;
	}
};

START_RFC_APPLICATION(MyApp)